<?php

include_once SITEAPPS_CLASS_DIR . "SiteAppsUtil.php";

class SiteAppsAdmin 
{
    
    public $plugin;
    
    public function __construct($plugin) 
    {
        $this->plugin = $plugin;
        
        $this->installation();
        
        //add All Scripts here
        $this->scripts();
        
        //Add All Styles here
        $this->styles();
        
        $this->registerCallBacks();
        
        $this->checkSegmentsUpdate();
        
        $this->renderPage();
        
        $this->widgets();
        
        $this->checkEmptySiteAppsId();
        
        $this->checkDependencies();
    }
    
    private function installation()
    {
        include_once SITEAPPS_CLASS_DIR . "SiteAppsPluginInstall.php";
        $install = new SiteAppsPluginInstall($this->plugin->requiredWpVersion, $this->plugin->options);
        register_activation_hook($this->plugin->path . '/' . SITEAPPS_PLUGIN_NAME . '.php', array($install, 'activate'));
        $this->plugin->options = $install->getOptions();
        //$install->checkVersion();
        if($this->plugin->options['id'] > 0) {
            $this->plugin->options['tag'] = $this->getTag($this->plugin->options['id'],$this->plugin->options['type']);
        }
    }
    
    private function widgets()
    {
        include_once SITEAPPS_CLASS_DIR . "widget/SiteAppsWidget.php";

        $widgetConfig = new SiteAppsWidget($this->plugin);
        $widgetConfig->enableSmartWidgets();
    }
    
    private function scripts()
    {
        add_action('admin_init', array($this, 'loadScripts'));
    }
    
    public function loadScripts() 
    {
        wp_register_script(SITEAPPS_PLUGIN_NAME . '_plugin_siteapps_config', plugins_url(SITEAPPS_PLUGIN_NAME) . '/js/siteapps-config.js', array('jquery'));
        wp_register_script(SITEAPPS_PLUGIN_NAME . '_jquery_livequery', plugins_url(SITEAPPS_PLUGIN_NAME) . '/js/jquery.livequery.js', array('jquery'));
        wp_register_script(SITEAPPS_PLUGIN_NAME . '_widgets_onload', plugins_url(SITEAPPS_PLUGIN_NAME) . '/js/widgets-onload.js', array('jquery'));
    }
    
    public function loadAdminScripts() 
    {
        wp_enqueue_script(SITEAPPS_PLUGIN_NAME.'_jquery_livequery');
        wp_enqueue_script(SITEAPPS_PLUGIN_NAME.'_plugin_siteapps_config');
    }
    
    private function styles()
    {
        add_action('admin_init', array($this, 'loadStyles'));    
        add_action('admin_print_styles', array($this, 'widgetStyles'));
    }
    
    public function loadStyles() 
    {
        wp_register_style(SITEAPPS_PLUGIN_NAME . '_style_admin', plugins_url(SITEAPPS_PLUGIN_NAME)  . '/css/style-admin.css');
        wp_register_style(SITEAPPS_PLUGIN_NAME . '_style_admin_widgets', plugins_url(SITEAPPS_PLUGIN_NAME) . '/css/style-admin-widgets.css');
    }
    
    public function widgetStyles()
    {
        if (strstr($_SERVER['REQUEST_URI'], 'options-general.php?page=siteapps')) {
            wp_enqueue_style(SITEAPPS_PLUGIN_NAME . '_style_admin');
        }
    }
    
    private function registerCallBacks()
    {
        add_action('admin_init', array($this, 'requestCallbacks'));
    }
    
    public function requestCallbacks()
    {
        if (isset($_POST['siteapps_save']) && $_POST['siteapps_save']) {
            $this->saveOptions();
        } elseif (isset($_POST['siteapps_refresh_data']) && $_POST['siteapps_refresh_data']) {
            $this->updateSegments();
        } elseif (isset($_POST['siteapps_reset']) && $_POST['siteapps_reset']) {
            $this->reset();
        }
    }
    
    public function reset()
    {
        check_admin_referer(SITEAPPS_PLUGIN_NAME);
        $this->plugin->options = SiteAppsPluginInstall::getDefaultOptions();
        update_option(SITEAPPS_PLUGIN_NAME, $this->plugin->options);
        add_action('admin_head', array($this, 'printTagOFF'));
    }
    
    private function saveOptions() 
    {
        check_admin_referer(SITEAPPS_PLUGIN_NAME);

        if (!current_user_can('manage_options')) {
            add_action('admin_head', array($this, 'printCannotEdit'));
            return;
        }

        if (isset($_POST['sa_id']) && $_POST['sa_id'] > 0) {
            $this->plugin->options['id']                  = (int) $_POST['sa_id'];
            $this->plugin->options['type']                = (int) $_POST['sa_tag_type'];
            $this->plugin->options['debug']               = (bool) $_POST['sa_debug'];
            $this->plugin->options['enable_smart_widgets'] = (bool) $_POST['sa_enable_smart_widgets'];            
            
            $exists = $this->updateSegments();

            if(!$exists) {
                $this->plugin->options['tag'] = '';
            } else {
                $this->plugin->options['tag'] = $this->getTag($this->plugin->options['id'],$this->plugin->options['type']);
                add_action('admin_head', array($this, 'printOKSiteAppsId'));
            }
            
            update_option(SITEAPPS_PLUGIN_NAME, $this->plugin->options);
        } elseif (isset($_POST['sa_id']) && $_POST['sa_id'] == 0) {
            add_action('admin_head', array($this, 'printInvalidSiteAppsId'));  
        }
    }
    
    private function renderPage()
    {
        add_action('admin_menu', array($this,'addToMenu'));
    }

    public function addToMenu() 
    {
        $optionsPage = add_options_page('SiteApps Plugin', 'SiteApps', 'manage_options', SITEAPPS_PLUGIN_NAME, array($this, "page"));
        add_action('admin_print_scripts-'.$optionsPage, array($this, 'loadAdminScripts'));
    }

    public function page() 
    {
        $this->plugin->options = get_option(SITEAPPS_PLUGIN_NAME);
        $options = $this->plugin->options;

        $saId               = $options['id'];
        $syncCheck          = ($options['type'] === 1)?'checked="true"':'';
        $asyncCheck         = ($options['type'] === 1)?'':'checked="true"';
        $smartWidgetCheck   = ($options['enable_smart_widgets'])?'checked="true"':'';
        $debugCheck         = ($options['debug'])?'checked="true"':'';
        $segments           = $options['segments'];

        require_once(SITEAPPS_VIEW_DIR.'admin/header.php');
        require_once(SITEAPPS_VIEW_DIR.'admin/sidebar.php');  
        require_once(SITEAPPS_VIEW_DIR.'admin/main.php');  
        require_once(SITEAPPS_VIEW_DIR.'admin/footer.php'); 
    }
   
    private function checkSegmentsUpdate() 
    {
        if (!isset($_POST['siteapps_save'])) {
            if ((time() - $this->plugin->options['last_updated']) > $this->plugin->options['refresh_interval']) {
                $this->updateSegments();
            }
        }
    }
    
    private function updateSegments()
    {
        if($this->plugin->options['id']) { 
            
            if(!$this->checkCurlIsLoaded()) {
                return true;
            }
            
            include_once SITEAPPS_CLASS_DIR . "SiteAppsAPI.php";
            $siteAppsAPI    = new SiteAppsAPI();
            $segments       = $siteAppsAPI->getSegmentsByClient($this->plugin->options['id'], $this->plugin->options['private_key'], $this->plugin->options['public_key']);
            if (count($segments) > 0) {
                $newSegments = array();
                foreach ($segments as $segment) {
                    $newSegments[$segment['friendly_name']] = $segment;
                }
                $this->plugin->options['segments'] = $newSegments;
                update_option(SITEAPPS_PLUGIN_NAME, $this->plugin->options);
                return true;
            } else {
                add_action('admin_head', array($this, 'printMsgToSiteAppsIdNotFound'));
                return false;
            }
        }
    }

    private function checkEmptySiteAppsId() 
    {
        $this->plugin->options = get_option(SITEAPPS_PLUGIN_NAME);
        
        if (!($this->plugin->options['id']) && !isset($_POST['sa_id'])) {
            add_action('admin_head', array($this, 'printMsgToSiteAppsId'));
        }
    }
    
    private function checkDependencies()
    {
        if (strstr($_SERVER['REQUEST_URI'], 'options-general.php?page=siteapps')) {
            if (!$this->checkCurlIsLoaded()) {
                add_action('admin_head', array($this, 'printCurlNotLoaded'));
            }
        }
    }
    
    private function checkCurlIsLoaded()
    {
        if  (in_array('curl', get_loaded_extensions())) {
            return true;
        } else {
            return false;
        }
    }

    public function printCurlNotLoaded()
    {
        SiteAppsUtil::notify('You don\'t have curl extension installed. Please, install it for use segments', true);
    }
    
    public function printCannotEdit()
    {
        SiteAppsUtil::notify('You cannot edit the SiteApps options.', true);
    }
    
    public function printInvalidSiteAppsId()
    {
        SiteAppsUtil::notify( 'Invalid SiteApps ID!', true);
    }

    public function printTagOFF()
    {
       SiteAppsUtil::notify('SiteApps Tag is <strong>OFF</strong>. You must enter your SiteApps Id to activate it.');      
    }
    
    public function printOKSiteAppsId()
    {
        SiteAppsUtil::notify('SiteApps settings updated, SiteApps Tag is <strong>ON</strong>.');
    }
    
    public function printMsgToSiteAppsId()
    {
        SiteAppsUtil::notify(sprintf(__('Please enter your SiteApps ID on the SiteApps %soptions page%s.'), '<a href="options-general.php?page=' . SITEAPPS_PLUGIN_NAME . '">', '</a>'));
    }
    
    public function printMsgToSiteAppsIdNotFound()
    {
        SiteAppsUtil::notify(sprintf(__('Your SiteApps ID was not found. Your Segments were not updated.')), true);
    }
    
    private function getTag($id, $type) 
    {
        if($type === 1) {
            return '<script type="text/javascript">
                    //<![CDATA[[
                    $SA={s:'.$id.'};
                    document.write(unescape("%3Cscript src=\'" + ("https:" == document.location.protocol ? "https://" + $SA.s + ".sa" : "http://" + $SA.s + ".a") + ".siteapps.com/" + $SA.s + ".js\' type=\'text/javascript\'%3E%3C/script%3E"));
                    //]]>
                    </script>';
        }
        return '<script type="text/javascript">
                //<![CDATA[[
                $SA={s:'.$id.',asynch:1};
                (function(){
                    var sa = document.createElement("script"); sa.type = "text/javascript"; sa.async = true;
                    sa.src = ("https:" == document.location.protocol ? "https://" + $SA.s + ".sa" : "http://" + $SA.s + ".a") + ".siteapps.com/" + $SA.s + ".js";
                    var t = document.getElementsByTagName("script")[0]; t.parentNode.insertBefore(sa, t);
                })();
                //]]>
                </script>'; 
    }
    
}

?>
